@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require "bundler/psyched_yaml"
+require "psych"
 require "bundler/vendored_fileutils"
 require "bundler/vendored_uri"
 require "digest"
@@ -113,6 +113,10 @@ RSpec.configure do |config|
     end
   end
 
+  config.before :each, :sudo => true do
+    Spec::Sudo.write_safe_config
+  end
+
   config.after :suite do
     FileUtils.rm_r Spec::Path.pristine_system_gem_path
   end
