@@ -1,5 +1,5 @@
 # frozen_string_literal: true
-require 'rubygems/test_case'
+require_relative 'helper'
 require 'rubygems/commands/push_command'
 
 class TestGemCommandsPushCommand < Gem::TestCase
@@ -123,7 +123,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
     Gem.configuration.disable_default_gem_server = true
     response = "You must specify a gem server"
 
-    assert_raises Gem::MockGemUi::TermError do
+    assert_raise Gem::MockGemUi::TermError do
       use_ui @ui do
         @cmd.send_gem(@path)
       end
@@ -155,7 +155,6 @@ class TestGemCommandsPushCommand < Gem::TestCase
       @host => @api_key,
     }
 
-    FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
     File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
@@ -190,7 +189,6 @@ class TestGemCommandsPushCommand < Gem::TestCase
       @host => @api_key,
     }
 
-    FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
     File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
@@ -232,7 +230,6 @@ class TestGemCommandsPushCommand < Gem::TestCase
       :rubygems_api_key => @api_key,
     }
 
-    FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
     File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
@@ -252,7 +249,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
 
     response = %(ERROR:  "#{@host}" is not allowed by the gemspec, which only allows "https://privategemserver.example")
 
-    assert_raises Gem::MockGemUi::TermError do
+    assert_raise Gem::MockGemUi::TermError do
       send_battery
     end
 
@@ -274,7 +271,6 @@ class TestGemCommandsPushCommand < Gem::TestCase
       @host => @api_key,
     }
 
-    FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
     File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
@@ -284,7 +280,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
 
     response = "ERROR:  \"#{@host}\" is not allowed by the gemspec, which only allows \"#{push_host}\""
 
-    assert_raises Gem::MockGemUi::TermError do
+    assert_raise Gem::MockGemUi::TermError do
       send_battery
     end
 
@@ -305,7 +301,6 @@ class TestGemCommandsPushCommand < Gem::TestCase
       host => api_key,
     }
 
-    FileUtils.mkdir_p File.dirname Gem.configuration.credentials_path
     File.open Gem.configuration.credentials_path, 'w' do |f|
       f.write keys.to_yaml
     end
@@ -332,7 +327,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
 
   def test_raises_error_with_no_arguments
     def @cmd.sign_in(*); end
-    assert_raises Gem::CommandLineError do
+    assert_raise Gem::CommandLineError do
       @cmd.execute
     end
   end
@@ -342,7 +337,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
     @fetcher.data["#{@host}/api/v1/gems"] = [response, 403, 'Forbidden']
     @cmd.instance_variable_set :@host, @host
 
-    assert_raises Gem::MockGemUi::TermError do
+    assert_raise Gem::MockGemUi::TermError do
       use_ui @ui do
         @cmd.send_gem(@path)
       end
@@ -392,7 +387,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
     @fetcher.data["#{Gem.host}/api/v1/gems"] = [response, 401, 'Unauthorized']
 
     @otp_ui = Gem::MockGemUi.new "111111\n"
-    assert_raises Gem::MockGemUi::TermError do
+    assert_raise Gem::MockGemUi::TermError do
       use_ui @otp_ui do
         @cmd.send_gem(@path)
       end
@@ -440,6 +435,7 @@ class TestGemCommandsPushCommand < Gem::TestCase
 
     response_mfa_enabled = "You have enabled multifactor authentication but your request doesn't have the correct OTP code. Please check it and retry."
     response_success     = 'Successfully registered gem: freewill (1.0.0)'
+    response_profile     = "mfa: disabled\n"
 
     @fetcher.data["#{@host}/api/v1/gems"] = [
       [response_success, 200, "OK"],
@@ -450,6 +446,10 @@ class TestGemCommandsPushCommand < Gem::TestCase
       ["", 200, "OK"],
     ]
 
+    @fetcher.data["#{@host}/api/v1/profile/me.yaml"] = [
+      [response_profile, 200, "OK"],
+    ]
+
     @cmd.instance_variable_set :@scope, :push_rubygem
     @cmd.options[:args] = [@path]
     @cmd.options[:host] = @host
