@@ -1,5 +1,5 @@
 # frozen_string_literal: true
-require 'rubygems/test_case'
+require_relative 'helper'
 require 'rubygems/dependency_installer'
 require 'rubygems/security'
 
@@ -44,7 +44,7 @@ class TestGemDependencyInstaller < Gem::TestCase
       s.add_development_dependency 'c'
     end
 
-    util_reset_gems
+    util_setup_spec_fetcher(@a1, @a1_pre, @b1, @d1)
   end
 
   def test_install
@@ -113,7 +113,7 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     dep = Gem::Dependency.new "p"
     inst = Gem::DependencyInstaller.new
-    assert_raises Gem::UnsatisfiableDependencyError do
+    assert_raise Gem::UnsatisfiableDependencyError do
       inst.install dep
     end
 
@@ -287,8 +287,6 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     @aa1, @aa1_gem = util_gem 'aa', '1'
 
-    util_reset_gems
-
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @aa1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
@@ -307,8 +305,6 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     @aa1, @aa1_gem = util_gem 'aa', '1'
 
-    util_reset_gems
-
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @aa1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
@@ -329,8 +325,6 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     @aa1, @aa1_gem = util_gem 'aa', '1'
 
-    util_reset_gems
-
     FileUtils.mv @a1_gem, @tempdir
     FileUtils.mv @aa1_gem, @tempdir
     FileUtils.mv @b1_gem, @tempdir
@@ -398,7 +392,7 @@ class TestGemDependencyInstaller < Gem::TestCase
 
     assert_equal %w[f-1], inst.installed_gems.map {|s| s.full_name }
 
-    assert_path_exists e1.extension_dir
+    assert_path_exist e1.extension_dir
   end
 
   def test_install_dependency_old
@@ -718,7 +712,7 @@ class TestGemDependencyInstaller < Gem::TestCase
     inst = nil
 
     Dir.chdir @tempdir do
-      e = assert_raises Gem::UnsatisfiableDependencyError do
+      e = assert_raise Gem::UnsatisfiableDependencyError do
         inst = Gem::DependencyInstaller.new :domain => :local
         inst.install 'b'
       end
@@ -889,7 +883,7 @@ class TestGemDependencyInstaller < Gem::TestCase
       policy = Gem::Security::HighSecurity
       inst = Gem::DependencyInstaller.new :security_policy => policy
 
-      e = assert_raises Gem::Security::Exception do
+      e = assert_raise Gem::Security::Exception do
         inst.install 'b'
       end
 
@@ -946,8 +940,33 @@ class TestGemDependencyInstaller < Gem::TestCase
     assert_equal %w[d-2], inst.installed_gems.map {|s| s.full_name }
   end
 
+  def test_install_legacy_spec_with_nil_required_ruby_version
+    path = File.expand_path 'data/null-required-ruby-version.gemspec.rz', __dir__
+    spec = Marshal.load Gem.read_binary(path)
+    def spec.validate(*args); end
+
+    util_build_gem spec
+
+    cache_file = File.join @tempdir, 'gems', "#{spec.original_name}.gem"
+    FileUtils.mkdir_p File.dirname cache_file
+    FileUtils.mv spec.cache_file, cache_file
+
+    util_setup_spec_fetcher spec
+
+    data = Gem.read_binary(cache_file)
+
+    @fetcher.data['http://gems.example.com/gems/activesupport-1.0.0.gem'] = data
+
+    dep = Gem::Dependency.new 'activesupport'
+
+    inst = Gem::DependencyInstaller.new
+    inst.install dep
+
+    assert_equal %w[activesupport-1.0.0], Gem::Specification.map(&:full_name)
+  end
+
   def test_install_legacy_spec_with_nil_required_rubygems_version
-    path = File.expand_path "../data/null-required-rubygems-version.gemspec.rz", __FILE__
+    path = File.expand_path 'data/null-required-rubygems-version.gemspec.rz', __dir__
     spec = Marshal.load Gem.read_binary(path)
     def spec.validate(*args); end
 
@@ -1131,16 +1150,6 @@ class TestGemDependencyInstaller < Gem::TestCase
     @d1, @d1_gem = util_gem 'd', '1'
     @d2, @d2_gem = util_gem 'd', '2'
 
-    util_reset_gems
-  end
-
-  def util_reset_gems
-    @a1     ||= nil
-    @b1     ||= nil
-    @a1_pre ||= nil
-    @d1     ||= nil
-    @d2     ||= nil
-
-    util_setup_spec_fetcher(*[@a1, @a1_pre, @b1, @d1, @d2].compact)
+    util_setup_spec_fetcher(@d1, @d2)
   end
 end
