@@ -1,11 +1,11 @@
 # frozen_string_literal: true
 
-require "rubygems/test_case"
+require_relative "helper"
 require "open3"
 
 class TestProjectSanity < Gem::TestCase
   def test_manifest_is_up_to_date
-    skip unless File.exist?(File.expand_path("../../../Rakefile", __FILE__))
+    pend unless File.exist?(File.expand_path('../../Rakefile', __dir__))
 
     _, status = Open3.capture2e("rake check_manifest")
 
@@ -13,7 +13,7 @@ class TestProjectSanity < Gem::TestCase
   end
 
   def test_require_rubygems_package
-    err, status = Open3.capture2e(*ruby_with_rubygems_in_load_path, "--disable-gems", "-e", "'require \"rubygems/package\"'")
+    err, status = Open3.capture2e(*ruby_with_rubygems_in_load_path, "--disable-gems", "-e", "require \"rubygems/package\"")
 
     assert status.success?, err
   end
