@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require "yaml"
+require "psych"
 
 class ChangelogEntry
   attr_reader :title, :template, :labels, :pull_request
@@ -55,14 +55,8 @@ class Changelog
   def initialize(file, version)
     @version = Gem::Version.new(version)
     @file = File.expand_path(file)
-    @config = YAML.load_file("#{File.dirname(file)}/.changelog.yml")
-    @level = if @version.segments[1..2] == [0, 0]
-               :major
-             elsif @version.segments[2] == 0
-               :minor
-             else
-               :patch
-             end
+    @config = Psych.load_file("#{File.dirname(file)}/.changelog.yml")
+    @level = @version.segments[2] != 0 ? :patch : :minor_or_major
   end
 
   def release_notes
@@ -235,8 +229,6 @@ class Changelog
   def relevant_changelog_label_mapping
     if @level == :patch
       changelog_label_mapping.slice(*patch_level_labels)
-    elsif @level == :minor
-      changelog_label_mapping.slice(*patch_level_labels + minor_level_labels)
     else
       changelog_label_mapping
     end
@@ -297,8 +289,4 @@ class Changelog
   def patch_level_labels
     @config["patch_level_labels"]
   end
-
-  def minor_level_labels
-    @config["minor_level_labels"]
-  end
 end
